==> worker_2.4.0.orig.tar.gz: worker-2.4.0/src/createsymlinkop.cc <==
/* createsymlinkop.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: createsymlinkop.cc,v 1.13 2002/03/16 23:27:35 ralf Exp $ */

#include "createsymlinkop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *CreateSymlinkOp::name="CreateSymlinkOp";

CreateSymlinkOp::CreateSymlinkOp(Worker *tworker):FunctionProto(tworker)
{
  same_dir=false;
  local=false;
  requestflags=false;
}

CreateSymlinkOp::~CreateSymlinkOp()
{
}

CreateSymlinkOp*
CreateSymlinkOp::duplicate()
{
  CreateSymlinkOp *ta=new CreateSymlinkOp(worker);
  ta->same_dir=same_dir;
  ta->local=local;
  ta->requestflags=requestflags;
  return ta;
}

bool
CreateSymlinkOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
CreateSymlinkOp::getName()
{
  return name;
}

int
CreateSymlinkOp::run(ActionMessage *msg)
{
  ListerMode *lm1;
  am=msg;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1=worker->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      endlister=worker->getOtherLister(startlister);
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodecreatesl();
    }
  } else normalmodecreatesl();
  return 0;
}

bool
CreateSymlinkOp::save(Datei *fh)
{
  int size=3*Datei::getUCharSize();
  fh->putInt(size);

  fh->putUChar((same_dir==true)?1:0);
  fh->putUChar((local==true)?1:0);
  fh->putUChar((requestflags==true)?1:0);
  return true;
}

int
CreateSymlinkOp::load(Datei *fh)
{
  int chunksize=fh->getInt();
  int cs=Datei::getUCharSize();
  if(chunksize>=(3*cs)) {
    if(fh->getUChar()==1) same_dir=true; else same_dir=false;
    if(fh->getUChar()==1) local=true; else local=false;
    if(fh->getUChar()==1) requestflags=true; else requestflags=false;
    chunksize-=3*cs;
  }
  while(chunksize>0) {
    fh->getUChar();
    chunksize--;
  }
  return true;
}

const char *
CreateSymlinkOp::getDescription()
{
  return catalog.getLocaleCom(36);
}

int
CreateSymlinkOp::normalmodecreatesl()
{
  NormalMode *nm1=NULL,
             *nm2=NULL;
  ListerMode *lm1=NULL;
  struct NM_specialsource *specialsource=NULL;
  bool cont=true;
  char *tstr,*dest=NULL;
  struct NM_createsymlinkorder cslorder;
  bool takesamedir;
  
  if(am->mode==am->AM_MODE_DNDACTION) {
    lm1=((DNDMsg*)am->extraInfo)->getSourceMode();
    if(lm1==NULL) return 1;
    if(lm1->isType("NormalMode")!=true) return 1;
    nm1=(NormalMode*)lm1;
    lm1=((DNDMsg*)am->extraInfo)->getDestMode();
    if(lm1!=NULL)
      if(lm1->isType("NormalMode")==true)
        nm2=(NormalMode*)lm1;
  } else {
    if(startlister==NULL) return 1;
    lm1=startlister->getActiveMode();
    if(lm1==NULL) return 1;
    if(lm1->isType("NormalMode")!=true) return 1;
    nm1=(NormalMode*)lm1;
    if(endlister!=NULL) {
      lm1=endlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          nm2=(NormalMode*)lm1;
    }
  }

  if(requestflags==true) {
    if(doconfigure(1)!=0) cont=false;
  } else {
    // set values in t* variables
    tsame_dir=same_dir;
    tlocal=local;
  }
  
  if(cont==true) {
    // detect what to copy
    if(am->mode==am->AM_MODE_ONLYACTIVE)
      cslorder.source=cslorder.NM_ONLYACTIVE;
    else if(am->mode==am->AM_MODE_DNDACTION) {
      // insert DND-element into list
      cslorder.source=cslorder.NM_SPECIAL;
      cslorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=((DNDMsg*)am->extraInfo)->getFE();
      specialsource->row=-1;
      cslorder.sources->addElement(specialsource);
    } else if(am->mode==am->AM_MODE_SPECIAL) {
      cslorder.source=cslorder.NM_SPECIAL;
      cslorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=(FileEntry*)am->extraInfo;
      specialsource->row=-1;
      cslorder.sources->addElement(specialsource);
    } else
      cslorder.source=cslorder.NM_ALLENTRIES;

    takesamedir=true;
    if(am->mode==am->AM_MODE_DNDACTION) {
      const char *ddstr=((DNDMsg*)am->extraInfo)->getDestDir();
      if(ddstr!=NULL) dest=dupstring(ddstr);
      else dest=NULL;
      takesamedir=false;
    } else if(tsame_dir==false) {
      if(nm2!=NULL) {
        tstr=nm2->getActualDir();
        if(tstr!=NULL) {
          dest=dupstring(tstr);
          takesamedir=false;
        }
      }
    }

    if(takesamedir==true) {
      tstr=nm1->getActualDir();
      if(tstr!=NULL)
        dest=dupstring(tstr);
    }
    
    if(dest!=NULL) {
      cslorder.local=tlocal;
      cslorder.destdir=dest;
      // now start copy process
      nm1->createsymlink(&cslorder);
    }
    if(dest!=NULL) _freesafe(dest);
    if(cslorder.source==cslorder.NM_SPECIAL) {
      if(specialsource!=NULL) _freesafe(specialsource);
      delete cslorder.sources;
    }
  }
  return 0;
}

int
CreateSymlinkOp::configure()
{
  return doconfigure(0);
}

int
CreateSymlinkOp::doconfigure(int mode)
{
  AGUIX *aguix=worker->getAGUIX();
  Button *okb,*cb;
  AWindow *win;
  ChooseButton *sdcb,*gcb,*rfcb=NULL;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  char *tstr;
  
  w=h=10;
  x=y=5;
  win=new AWindow(aguix);

  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(36))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(36));
  win->create(NULL,10,10,w,h,0,tstr);
  _freesafe(tstr);

  sdcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(same_dir==true)?1:0,
                                                catalog.getLocale(315),LABEL_RIGHT,1,0));
  y+=sdcb->getHeight()+5;
  tw=sdcb->getWidth()+10;
  if(tw>w) w=tw;

  gcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(local==true)?1:0,
                                               catalog.getLocale(316),LABEL_RIGHT,1,0));
  y+=gcb->getHeight()+5;
  tw=gcb->getWidth()+10;
  if(tw>w) w=tw;

  if(mode==0) {
    rfcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(requestflags==true)?1:0,
                                                 catalog.getLocale(294),LABEL_RIGHT,1,0));
    y+=rfcb->getHeight()+5;
    tw=rfcb->getWidth()+10;
    if(tw>w) w=tw;
  }

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;
  
  h=y;
  win->resize(w,h);
  win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
        case AG_KEYRELEASED:
          if(win->isParent(msg->key.window,false)==true) {
            switch(msg->key.key) {
              case XK_1:
                sdcb->setState((sdcb->getState()==1)?0:1);
                break;
              case XK_2:
                gcb->setState((gcb->getState()==1)?0:1);
                break;
              case XK_Return:
                endmode=0;
                break;
              case XK_Escape:
                endmode=1;
                break;
            }
          }
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    if(mode==1) {
      tsame_dir=(sdcb->getState()==1)?true:false;
      tlocal=(gcb->getState()==1)?true:false;
    } else {
      same_dir=(sdcb->getState()==1)?true:false;
      local=(gcb->getState()==1)?true:false;
      requestflags=(rfcb->getState()==1)?true:false;
    }
  }
  
  win->close();
  delete win;

  return endmode;
}

void CreateSymlinkOp::setSameDir(bool nv)
{
  same_dir=nv;
}

void CreateSymlinkOp::setLocal(bool nv)
{
  local=nv;
}

void CreateSymlinkOp::setRequestFlags(bool nv)
{
  requestflags=nv;
}

==> worker_2.8.4.orig.tar.gz: worker-2.8.4/src/createsymlinkop.cc <==
/* createsymlinkop.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: createsymlinkop.cc,v 1.22 2004/01/20 22:20:06 ralf Exp $ */

#include "createsymlinkop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *CreateSymlinkOp::name="CreateSymlinkOp";

CreateSymlinkOp::CreateSymlinkOp() : FunctionProto()
{
  same_dir=false;
  local=false;
  requestflags=false;
  hasConfigure = true;
}

CreateSymlinkOp::~CreateSymlinkOp()
{
}

CreateSymlinkOp*
CreateSymlinkOp::duplicate() const
{
  CreateSymlinkOp *ta=new CreateSymlinkOp();
  ta->same_dir=same_dir;
  ta->local=local;
  ta->requestflags=requestflags;
  return ta;
}

bool
CreateSymlinkOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
CreateSymlinkOp::getName()
{
  return name;
}

int
CreateSymlinkOp::run(ActionMessage *msg)
{
  ListerMode *lm1;
  am=msg;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1 = msg->getWorker()->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      endlister = msg->getWorker()->getOtherLister(startlister);
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodecreatesl();
    }
  } else normalmodecreatesl();
  return 0;
}

bool
CreateSymlinkOp::save(Datei *fh)
{
  if ( fh == NULL ) return false;
  WConfig::configPutPairBool( fh, "samedir", same_dir );
  WConfig::configPutPairBool( fh, "relative", local );
  WConfig::configPutPairBool( fh, "requestflags", requestflags );
  return true;
}

const char *
CreateSymlinkOp::getDescription()
{
  return catalog.getLocaleCom(36);
}

int
CreateSymlinkOp::normalmodecreatesl()
{
  NormalMode *nm1=NULL,
             *nm2=NULL;
  ListerMode *lm1=NULL;
  struct NM_specialsource *specialsource=NULL;
  bool cont=true;
  char *tstr,*dest=NULL;
  struct NM_createsymlinkorder cslorder;
  bool takesamedir;
  
  if(am->mode==am->AM_MODE_DNDACTION) {
    lm1=((DNDMsg*)am->extraInfo)->getSourceMode();
    if(lm1==NULL) return 1;
    if(lm1->isType("NormalMode")!=true) return 1;
    nm1=(NormalMode*)lm1;
    lm1=((DNDMsg*)am->extraInfo)->getDestMode();
    if(lm1!=NULL)
      if(lm1->isType("NormalMode")==true)
        nm2=(NormalMode*)lm1;
  } else {
    if(startlister==NULL) return 1;
    lm1=startlister->getActiveMode();
    if(lm1==NULL) return 1;
    if(lm1->isType("NormalMode")!=true) return 1;
    nm1=(NormalMode*)lm1;
    if(endlister!=NULL) {
      lm1=endlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          nm2=(NormalMode*)lm1;
    }
  }

  if(requestflags==true) {
    if(doconfigure(1)!=0) cont=false;
  } else {
    // set values in t* variables
    tsame_dir=same_dir;
    tlocal=local;
  }
  
  if(cont==true) {
    memset( &cslorder, 0, sizeof( cslorder ) );
    // detect what to copy
    if(am->mode==am->AM_MODE_ONLYACTIVE)
      cslorder.source=cslorder.NM_ONLYACTIVE;
    else if(am->mode==am->AM_MODE_DNDACTION) {
      // insert DND-element into list
      cslorder.source=cslorder.NM_SPECIAL;
      cslorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=((DNDMsg*)am->extraInfo)->getFE();
      specialsource->row=-1;
      cslorder.sources->addElement(specialsource);
    } else if(am->mode==am->AM_MODE_SPECIAL) {
      cslorder.source=cslorder.NM_SPECIAL;
      cslorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=(FileEntry*)am->extraInfo;
      specialsource->row=-1;
      cslorder.sources->addElement(specialsource);
    } else
      cslorder.source=cslorder.NM_ALLENTRIES;

    takesamedir=true;
    if(am->mode==am->AM_MODE_DNDACTION) {
      const char *ddstr=((DNDMsg*)am->extraInfo)->getDestDir();
      if(ddstr!=NULL) dest=dupstring(ddstr);
      else dest=NULL;
      takesamedir=false;
    } else if(tsame_dir==false) {
      if(nm2!=NULL) {
        tstr = nm2->getCurrentDir();
        if(tstr!=NULL) {
          dest=dupstring(tstr);
          takesamedir=false;
        }
      }
    }

    if(takesamedir==true) {
      tstr = nm1->getCurrentDir();
      if(tstr!=NULL)
        dest=dupstring(tstr);
    }
    
    if(dest!=NULL) {
      cslorder.local=tlocal;
      cslorder.destdir=dest;
      // now start copy process
      nm1->createsymlink(&cslorder);
    }
    if(dest!=NULL) _freesafe(dest);
    if(cslorder.source==cslorder.NM_SPECIAL) {
      if(specialsource!=NULL) _freesafe(specialsource);
      delete cslorder.sources;
    }
  }
  return 0;
}

int
CreateSymlinkOp::configure()
{
  return doconfigure(0);
}

int
CreateSymlinkOp::doconfigure(int mode)
{
  AGUIX *aguix = Worker::getAGUIX();
  Button *okb,*cb;
  AWindow *win;
  ChooseButton *sdcb,*gcb,*rfcb=NULL;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  char *tstr;
  
  w=h=10;
  x=y=5;
  win=new AWindow(aguix);

  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(36))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(36));
  win->create(NULL,10,10,w,h,0,tstr);
  _freesafe(tstr);

  Text *tt1 = (Text*)win->add( new Text( aguix, x, y, catalog.getLocaleCom( 36 ), 1 ) );
  y += tt1->getHeight() + 5;

  sdcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(same_dir==true)?1:0,
                                                catalog.getLocale(315),LABEL_RIGHT,1,0));
  y+=sdcb->getHeight()+5;
  tw=sdcb->getWidth()+10;
  if(tw>w) w=tw;

  gcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(local==true)?1:0,
                                               catalog.getLocale(533),LABEL_RIGHT,1,0));
  y+=gcb->getHeight()+5;
  tw=gcb->getWidth()+10;
  if(tw>w) w=tw;

  if(mode==0) {
    rfcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(requestflags==true)?1:0,
                                                 catalog.getLocale(294),LABEL_RIGHT,1,0));
    y+=rfcb->getHeight()+5;
    tw=rfcb->getWidth()+10;
    if(tw>w) w=tw;
  }

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;
  
  h=y;
  okb->takeFocus();
  win->setDoTabCycling( true );
  win->resize(w,h);
  win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
        case AG_KEYPRESSED:
          if(win->isParent(msg->key.window,false)==true) {
            switch(msg->key.key) {
              case XK_1:
                sdcb->setState((sdcb->getState()==1)?0:1);
                break;
              case XK_2:
                gcb->setState((gcb->getState()==1)?0:1);
                break;
              case XK_Return:
                if ( cb->hasFocus() == false ) {
                  endmode=0;
                }
                break;
              case XK_Escape:
                endmode=1;
                break;
            }
          }
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    if(mode==1) {
      tsame_dir=(sdcb->getState()==1)?true:false;
      tlocal=(gcb->getState()==1)?true:false;
    } else {
      same_dir=(sdcb->getState()==1)?true:false;
      local=(gcb->getState()==1)?true:false;
      requestflags=(rfcb->getState()==1)?true:false;
    }
  }
  
  win->close();
  delete win;

  return endmode;
}

void CreateSymlinkOp::setSameDir(bool nv)
{
  same_dir=nv;
}

void CreateSymlinkOp::setLocal(bool nv)
{
  local=nv;
}

void CreateSymlinkOp::setRequestFlags(bool nv)
{
  requestflags=nv;
}
